//----------------------------------------------------------------------
//MODEULE: uart0.c
//COMMON DESCRIPTION:
//  This module provide support of internal UART0
//----------------------------------------------------------------------
#include "uart0.h"
#include <avr/signal.h>
#include <avr/pgmspace.h>
//
#ifdef UART0_ENABLED
  #ifdef UART_BUFFERS_ENABLED
  //----------------------------------------------------------------------
  // RX Buffer
  unsigned char uart0_rxbuf[UART_BUFFERS_LEN];
  unsigned char uart0_brxbuf; // begin of RX buffer
  unsigned char uart0_erxbuf; // end of RX buffer
  //----------------------------------------------------------------------
  // TX Buffer
  unsigned char uart0_txbuf[UART_BUFFERS_LEN];
  unsigned char uart0_btxbuf; // begin of TX buffer
  unsigned char uart0_etxbuf; // end of TX buffer
  //----------------------------------------------------------------------
  // Functions of clearing buffers
  void uart0_clear_rx_buf(){uart0_brxbuf=uart0_erxbuf=0;}
  void uart0_clear_tx_buf(){uart0_btxbuf=uart0_etxbuf=0;}
  //----------------------------------------------------------------------
  // Returned TRUE if TX buffer is empty
  unsigned char uart0_txbuf_empty()
    {unsigned char ret=0;
     if(uart0_etxbuf==uart0_btxbuf){ret=0x01;}
     return(ret);
    }
  // Returned TRUE if RX buffer is empty
  unsigned char uart0_rxbuf_empty()
    {unsigned char ret=0;
     if(uart0_erxbuf==uart0_brxbuf){ret=0x01;}
     return(ret);
    }
  //----------------------------------------------------------------------
  // Function of cyclic increment
  unsigned char uart0_pcinc(unsigned char pc)
    {pc++;
     if(pc>=UART_BUFFERS_LEN){pc=0;}
     return(pc);
    }
  //----------------------------------------------------------------------
  // Interrupt handlers
  //----------------------------------------------------------------------
  // RX Complete interrupt handler
  SIGNAL (SIG_UART_RECV)
    {if(uart0_pcinc(uart0_erxbuf)!=uart0_brxbuf)
       {// Reading byte
        uart0_rxbuf[uart0_erxbuf]=input(UDR);
	uart0_erxbuf=uart0_pcinc(uart0_erxbuf);
       }
    }
  //----------------------------------------------------------------------
  // TX Complete interrupt handler
  SIGNAL (SIG_UART_TRANS)
    {if(uart0_etxbuf!=uart0_btxbuf)
       {// Writting byte
        output(UDR,uart0_txbuf[uart0_btxbuf]);_NOP();_NOP();
        uart0_btxbuf=uart0_pcinc(uart0_btxbuf);
       }
    }
  #endif
  //----------------------------------------------------------------------
  // Init UART0
  void uart0_init()
    {// Init of UART0
     #ifndef UART_SPD_CONST
       uart0_setspeed(9600);
     #else
       output(UBRR,(unsigned char)((long)F_CLK/(16*(long)UART_SPD_CONST))-1);_NOP();_NOP();
     #endif
     // Init buffers
     #ifdef UART_BUFFERS_ENABLED
     uart0_clear_rx_buf();
     uart0_clear_tx_buf();
     // init UCR
     output(UCR,(1<<RXEN) | (1<<TXEN) | (1<<RXCIE) | (1<<TXCIE));_NOP();_NOP();
     #else
     // init UCR
     output(UCR,(1<<RXEN) | (1<<TXEN));_NOP();_NOP();
     #endif     
    }
  //----------------------------------------------------------------------
  // Define procedures
  #ifndef UART_SPD_CONST
    // Set speed (speed in Bits/sec)
    void uart0_setspeed(unsigned int speed)
      {output(UBRR,(unsigned char)((long)F_CLK/(16*(long)speed))-1);_NOP();_NOP();
      }
  #endif
  //----------------------------------------------------------------------
  // Write to uart0 (or writing-buffer)
  unsigned char uart0_wr(char* c)
    {unsigned char fret;
     #ifdef UART_BUFFERS_ENABLED
     unsigned char fstbyte=0;
     if(uart0_etxbuf==uart0_btxbuf){fstbyte=1;}
     // Write with buffer enable
     if(uart0_pcinc(uart0_etxbuf)!=uart0_btxbuf)
       {// Writting byte
        uart0_txbuf[uart0_etxbuf]=*c;
	uart0_etxbuf=uart0_pcinc(uart0_etxbuf);
	// Send first byte
	if((input(USR) & (1 << UDRE)) && fstbyte)
          {output(UDR,*c);_NOP();_NOP();
	   uart0_btxbuf=uart0_pcinc(uart0_btxbuf);	  
          }
	 fret=1;
       }
     else
       {// Buffer is full, can't write
        fret=0;
       }
     #endif
     // Write without buffer enable
     #ifndef UART_BUFFERS_ENABLED
     if(input(USR) & (1 << UDRE))
       {output(UDR,*c);_NOP();_NOP();
        fret=1;
       }
     else{fret=0;}
     #endif
     return(fret);
    }
  //----------------------------------------------------------------------
  // Read from uart0 (or reading-buffer)
  unsigned char  uart0_rd(char* c)
    {unsigned char fret; 
     #ifdef UART_BUFFERS_ENABLED
     // Read with buffer enable
     if(uart0_erxbuf!=uart0_brxbuf)
       {*c=uart0_rxbuf[uart0_brxbuf];
        uart0_brxbuf=uart0_pcinc(uart0_brxbuf);
	fret=1;
       }
     else
       {// Buffer is empty
        fret=0;
       }
     #endif
     // Read without buffer enable
     #ifndef UART_BUFFERS_ENABLED
     if(input(USR) & (1 << RXC))
       {*c=input(UDR);
        fret=1;
       }
     else{fret=0;}
     #endif
     return(fret);
    }
  //----------------------------------------------------------------------
  // Uart Strings Support
  //----------------------------------------------------------------------
  #ifdef UART_STR_ENABLED
    // Write string from RAM
    void uart_wrstr(unsigned char* s)
      {int i=0;
       while (s[i])
         {while(!uart0_wr(&s[i])){}
	  i++;
	 }
      }
    // Write string from Program Memory
    void uart_p_wrstr(unsigned char* s)
      {int i=0;
       char ch;
       while (1)
         {ch=PRG_RDB(s+i);
	  if(!ch){break;}
	  while(!uart0_wr(&ch)){}
	  i++;
	 }
      }
    // Write number
    void uart_wrint(unsigned int n)
      {unsigned char ch;
       unsigned int  d=10000;
       unsigned char k;
       //
       while (d)
         {k=n/d;
          ch=k+'0';
          n=n-d*k;
	  d=d/10;
	  while(!uart0_wr(&ch)){}
         }
       //
      }
    //
  #endif
#endif
